/****************************************************************************/
/******** FSTREE.C - File system tree - Ren Lang - Copyright (C) 1999 ******/
/****************************************************************************/
/******** rekursive Ermittlung der Gre von Unterverzeichnissen (Anzahl ****/
/******** der Dateien, belegter Plattenplatz und Verzeichnisstruktur) *******/
/****************************************************************************/

/****************************************************************************/
/******** bentigte Header-Dateien ******************************************/
/****************************************************************************/
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
#include <string.h>
#include <stdlib.h>
#include "fstree.h"

/****************************************************************************/
/******** bergebene Zeichenkette in Grobuchstaben konvertieren ************/
/****************************************************************************/
void convert_parm (char *parm_ptr)
{
    char *ptr;

    ptr = parm_ptr;
    while (*ptr) {
	if (islower (*ptr)) *ptr = toupper (*ptr);
	ptr++;
    }
}

/****************************************************************************/
/******** long-Zahl in eine Zeichenkette umwandeln und dabei in Dreier- *****/
/******** gruppen mit Punkten separieren ************************************/
/****************************************************************************/
void printlong (unsigned long zahl)
{
    char zahlfeld1[12], zahlfeld2[15];
    int  i, j;

    sprintf (zahlfeld1, "%11ld", zahl);
    i = 14; j = 11;
    strcpy (zahlfeld2, "              ");
    while (j > 0) {
	if (zahlfeld1[j] != ' ') zahlfeld2[i--] = zahlfeld1[j];
	j--;
	if (! ((i+2) % 4) && (zahlfeld1[j] != ' ')) zahlfeld2[i--] = '.';
    }
    printf ("%s", zahlfeld2);
}

/****************************************************************************/
/******** Anzeigen der Versionsinformation **********************************/
/****************************************************************************/
void show_version_info (char *pgm_name)
{
    printf (fstree_version, pgm_name);
    exit (3);
}

/****************************************************************************/
/******** Anzeigen des Hilfetextes ******************************************/
/****************************************************************************/
void show_help_text ()
{
    printf (fstree_help1);
    printf (fstree_help2);
    printf (fstree_help3);
    exit (3);
}

/****************************************************************************/
/******** Ermitteln von Dateianzahl und Gesamtgre von Unterverzeichnissen */
/****************************************************************************/
summary fstreesize (boolean first_run, boolean second_run, unsigned char flags,
		  char *path, char *fstree_location)
{
    DIR           *directory;
    struct dirent *dir_entry;
    struct stat    fileinfo;
    char	   current_dir[MAXPATHLEN];
    summary        total_summary, cur_summary, help_summary;

    /******** Initialisierungen *********************************************/
    total_summary.size    = cur_summary.size    = 0L;
    total_summary.dirsize = cur_summary.dirsize = 0L; 
    total_summary.count   = cur_summary.count   = 0L;
    total_summary.subdir  = cur_summary.subdir  = 0;

    /******** aktuelles Verzeichnis ffnen **********************************/
    if (chdir (path) < 0) printf (fstree_fault1, fstree_location, path);
    else if ((directory = opendir (".")) == NULL)
	printf (fstree_fault1, fstree_location, path);
    else {
	dir_entry = readdir (directory);

	/******** alle Dateien und Verzeichnisse des aktuellen Verzeich- ****/
	/******** nisses werden verarbeiten (auer den Spezialverzeich- *****/
	/******** nissen), Ermittlung der Daten *****************************/
	while (directory != NULL && dir_entry != NULL) {
            if (lstat (dir_entry->d_name, &fileinfo) < 0) continue;
	    if (S_ISDIR (fileinfo.st_mode) != 0) {
		if ((strcmp (dir_entry->d_name, ".")  != 0) &&
		    (strcmp (dir_entry->d_name, "..") != 0)) {
		    if (first_run) help_summary = fstreesize
			(FALSE, TRUE, flags, dir_entry->d_name, fstree_location);
		    else help_summary = fstreesize
			(FALSE, FALSE, flags, dir_entry->d_name, fstree_location);
		    total_summary.subdir  += help_summary.subdir + 1;
		    total_summary.count   += help_summary.count;
		    total_summary.size    += help_summary.size;
		    total_summary.dirsize += fileinfo.st_size;
		}
	    }
	    else {
		cur_summary.size += fileinfo.st_size;
		cur_summary.count++;
	    }
	    dir_entry = readdir (directory);
	}
	closedir (directory);

	/******** wenn das TOP-Flag gesetzt ist, werden nur die Daten der ***/
	/******** Unterverzeichnisse der ersten Ebene summiert ausgegeben ***/
	if (second_run && (flags & FSTREE_FLAG_TOP)) {
	    printlong (total_summary.size + cur_summary.size);
	    printf (fstree_data, (total_summary.count + cur_summary.count),
		getcwd (current_dir, MAXPATHLEN));
	}

	/******** fr jedes Verzeichnis die Summenzeile ausgeben (auer *****/
	/******** wenn das TOP-Flag gesetzt ist) ****************************/
	if (! (flags & FSTREE_FLAG_TOP)) {
	    printlong (cur_summary.size);
	    printf (fstree_data, cur_summary.count,
		getcwd (current_dir, MAXPATHLEN));
	}

	/******** Abschluarbeiten und Parameter-Rckgabe *******************/
	if (strcmp (path, ".") != 0) chdir ("..");
    }
    total_summary.size   += cur_summary.size;
    total_summary.count  += cur_summary.count;
    total_summary.subdir += cur_summary.subdir;
    return (total_summary);
}

/****************************************************************************/
/******** Analyse der bergebenen Parameter und deren Aufbereitung **********/
/****************************************************************************/
int parse_parm (char *parm, unsigned char *flags)
{
    char *ptr;
    int   i = 1;

    /******** Analyse der bergebenen Parameter und deren Aufbereitung ******/
    ptr = parm;
    switch (*ptr++) {
    case '-' :
	convert_parm (parm);
	switch (*ptr) {
	case '-' :

            /******** Schlsselwort --HELP **********************************/
	    if (strcmp (parm, "--HELP") == 0) {
		if (*flags & FSTREE_FLAG_HELP) i = -1;
		else *flags |= FSTREE_FLAG_HELP;
	    }

            /******** Schlsselwort --VERSION *******************************/
	    else if (strcmp (parm, "--VERSION") == 0) {
		if (*flags & FSTREE_FLAG_VER) i = -1;
		else *flags |= FSTREE_FLAG_VER;
	    }

            /******** Schlsselwort --SUM ***********************************/
	    else if (strcmp (parm, "--SUM") == 0) {
		if (*flags & FSTREE_FLAG_SUM) i = -1;
		else *flags |= FSTREE_FLAG_SUM;
	    }

            /******** Schlsselwort --TOP ***********************************/
	    else if (strcmp (parm, "--TOP") == 0) {
		if (*flags & FSTREE_FLAG_TOP) i = -1;
		else *flags |= FSTREE_FLAG_TOP;
	    }

            /******** ungltiges Schlsselwort gefunden *********************/
	    else i = -2;
	    break;

	default  :
	    while (*ptr != '\0') {
        	/******** Flag S (alternativ zu --SUM) **********************/
		if (*ptr == 'S') {
		    if (*flags & FSTREE_FLAG_SUM) i = -1;
		    else *flags |= FSTREE_FLAG_SUM;
		}

        	/******** Flag T (alternativ zu --TOP) **********************/
		else if (*ptr == 'T') {
		    if (*flags & FSTREE_FLAG_TOP) i = -1;
		    else *flags |= FSTREE_FLAG_TOP;
		}

        	/******** ungltiges Flag gefunden **************************/
		else i = -2;
		ptr++;
	    }
	    break;
	    
	}
	break;

    /******** aktueller Parameter kann Verzeichnis sein *********************/
    default  :
	i = 0;
	break;
    }

    return (i);
}


/****************************************************************************/
/******** Hauptprogramm *****************************************************/
/****************************************************************************/
int main (int argc, char *argv[])
{
    char          directory[MAXPATHLEN], save_dir[MAXPATHLEN];
    unsigned char flags = FALSE;
    summary       summary_data;
    int           i, j;

    /******** Initialisierungen (Default-Startverzeichnis) ******************/
    strcpy (directory, ".");
    strcpy (save_dir, ".");
    j = argc;

    /******** Parameter auswerten *******************************************/
    while (--j > 0) {
	i = parse_parm (argv[j], &flags);
	switch (i) {
	case -2 : 
	    printf (fstree_fault2, argv[0], argv[j]);
	    show_help_text ();
	    exit (-1);
	    break;
	case -1 : 
	    printf (fstree_fault3, argv[0], argv[j]);
	    show_help_text ();
	    exit (-1);
	    break;
	case  0 :
	    strncpy (directory, argv[j], MAXPATHLEN);
	    getcwd (save_dir, MAXPATHLEN);
	    break;
	default : break;
	}
    }

    /******** eigentliche Programmaktionen **********************************/
    if (flags & FSTREE_FLAG_HELP) show_help_text ();
    else if (flags & FSTREE_FLAG_VER) show_version_info (argv[0]);
    else summary_data = fstreesize (TRUE, FALSE, flags, directory, argv[0]);

    /******** falls Summenzeile gefordert ist, dann auch ausgeben ***********/
    if (flags & FSTREE_FLAG_SUM) {
	printf    (fstree_sum1, getcwd (directory, MAXPATHLEN));
	printlong (summary_data.size);
	printf    (fstree_sum2, summary_data.count, summary_data.subdir);
	printlong (summary_data.dirsize);
	printf    (fstree_sum3);
    }

    /******** Abschluarbeiten **********************************************/
    chdir (save_dir);
    if      (summary_data.subdir >  0 && summary_data.count >  0 && summary_data.size >  0) exit (0);
    else if (summary_data.subdir >  0 && summary_data.count >  0 && summary_data.size == 0) exit (1);
    else if (summary_data.subdir >  0 && summary_data.count == 0 && summary_data.size == 0) exit (2);
    else if (summary_data.subdir == 0 && summary_data.count >  0 && summary_data.size >  0) exit (3);
    else if (summary_data.subdir == 0 && summary_data.count >  0 && summary_data.size == 0) exit (4);
    else exit (5);
}
