/****************************************************************************/
/******** FSTREE.H - File system tree - Ren Lang - Copyright (C) 1999 ******/
/****************************************************************************/
/******** recursively determination of the size of directorys (counts *******/
/******** the number of files, used disk space and shows the structure ******/
/******** of the filesystem hierarchy) **************************************/
/****************************************************************************/
/******** header file for definitions and declarations (English) ************/
/****************************************************************************/

/****************************************************************************/
/******** necessary definitions *********************************************/
/****************************************************************************/
#define FALSE              (0x00)
#define TRUE               (0x01)

#define FSTREE_FLAG_SUM    (0x01)
#define FSTREE_FLAG_TOP    (0x02)
#define FSTREE_FLAG_VER    (0x04)
#define FSTREE_FLAG_HELP   (0x08)

/****************************************************************************/
/******** used self defined data types **************************************/
/****************************************************************************/
typedef unsigned char boolean;
typedef struct {
    unsigned long size, dirsize;
    unsigned long count;
    unsigned int  subdir;
} summary;

/****************************************************************************/
/******** used text strings (error messages and constants) ******************/
/****************************************************************************/
char *fstree_help1   = "\n  Shows the filesystem hierarchy and some directory information.\n";
char *fstree_help2   = "  The overall size and the count of files will be written recursively.\n\n";
char *fstree_help3   = "  usage:  fstree [-st] [--sum] [--top] [--version] [--help] [<directory>]\n\n";

char *fstree_fault1  = "%s: no directory or permission denied: %s\n";
char *fstree_fault2  = "%s: illegal option: %s\n";
char *fstree_fault3  = "%s: option %s or his equivalent was given multiple\n";

char *fstree_version = "%s - version 1.00\n";
char *fstree_data    = " bytes in %6d files in %s\n";

char *fstree_sum1    = "\nSummary for directory %s\n";
char *fstree_sum2    = " bytes in %6d files in %-d subdirectories\n";
char *fstree_sum3    = " bytes in directory entries\n\n";

/****************************************************************************/
/******** prototypes of the internal functions ******************************/
/****************************************************************************/
void    show_version_info (char *);
void    show_help_text    (void);
void    convert_parm      (char *);
int     parse_parm        (char *, unsigned char *);
void    printlong         (unsigned long);
summary fstreesize        (boolean, boolean, unsigned char, char *, char *);

